#!/bin/bash
###
### This routine calls upon AWK, BASH, PSQL, and IDL scripts to generate
### the analytical results for the paper:
### "Habitat and Tectonic Controls of Marine Biogeography"
### You will need to use your own k-means clustering software (see lines
### 73-80)
###
table=mdiv_patch
tmax=60
fname1=clust/obs.raw.patch
fname2=idl/mdiv/data/gnames.dat
fname3=idl/mdiv/data/mdiv_patch.dat
fname4=idl/mdiv/out/mdiv_1.out
fname5=idl/mdiv/out/mdiv_2.out
fname6=idl/mdiv/out/mdiv_icorr_1.out
fname7=idl/mdiv/out/mdiv_icorr_2.out
fname8=idl/mdiv/out/mdiv_icorr_3.out
fname9=idl/mdiv/out/mdiv_icorr_4.out
fname10=idl/mdiv/out/mdiv_icorr_5.out
fname11=idl/mdiv/out/mdiv_icorr_6.out
fname12=idl/mdiv/out/mdiv_icorr_7.out
fname13=idl/mdiv/out/mdiv_cen_corr_1.out
fname14=idl/mdiv/out/mdiv_cen_corr_2.out
fname15=idl/mdiv/out/mdiv_cen_corr_3.out
###
echo `date`
pushd clust/
echo Cleaning up files
rm -f clusters.* obs.raw* obs.bin* seeds.out.*
popd
rm -f ${fname2} ${fname3} ${fname4} ${fname5} ${fname6} ${fname7} ${fname8} ${fname9} ${fname10} ${fname11} ${fname12} ${fname13} ${fname14} ${fname15}
### Create database sed_rock
createdb sed_rock
echo Creating table mdiv_lith, and mdiv_env, then table mdiv, which depends on mdiv_lith and mdiv_env.
psql sed_rock < mdiv_lith.sql
psql sed_rock < mdiv_env.sql
psql sed_rock < mdiv.sql
psql sed_rock < mdiv_meso.sql
psql sed_rock < mdiv_icorr.sql
psql sed_rock < mdiv_cen_corr.sql
echo Done
echo ""
### Fix up the .csv file and extract the desired fields
echo Parsing the initial data.
./fix_fields.bash | ./extract_fields.bash > mdiv.db; sed -i 's/claystone\/claystone/claystone_only/' mdiv.db
echo Done
echo ""
### Create SQL commands and load the fields into tables mdiv_lith and mdiv
echo Populating table mdiv_lith.
awk -f load_lith.awk mdiv_lith.db | psql sed_rock
echo Done populating table mdiv_lith.
echo ""
echo Populating table mdiv_env.
awk -f load_env.awk mdiv_env.db | psql sed_rock
echo Done populating table mdiv_env.
echo ""
echo Populating table mdiv.
awk -f load.awk mdiv.db | psql sed_rock
echo Done populating table mdiv.
echo ""
### Check number of records in mdiv table
echo "Verifing table mdiv. The number of entries in mdiv is: `psql sed_rock -tc 'select count(*) from mdiv'`"
### and compare with the extracted data file
echo "The number of entries above should match this number: `wc -l mdiv.db | awk '{print $1}'`"
echo ""
### Run the k-means clustering algorithm for all cluster sizes, 
### then parse the cluster assignments and within cluster variance 
### from the clustering runs, which will be used in a scree plot. This 
### will be done later.
echo Parse raw data for clustering
psql sed_rock -tc "select pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1 order by ma_mid" | head -n-1  | sed 's/|//g' > ${fname1}
#echo 'Starting k-means clustering for n=5...100 clusters'
#pushd clust/
#rm -f *.log output.dat.patch.* pseeds2b.input* seeds.out.patch.* cluster*
#./loop.bash
#echo 'Starting k-means clustering for n=40 clusters, trials=1...100'
#./loop_rand_seed.bash
#popd
#echo 'Done clustering'
echo Appending cluster assignments to PSQL table ${table}
echo Parsing in cluster assignments for n=5...100 clusters
psql sed_rock -tc "select ma_mid,class_name,genus_name,species_name,mdiv.lithology,mdiv.environment,latdec,londec,geom,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1 order by ma_mid" | head -n-1 | sed 's/^ * //g;s/ * | * /|/g;s/| * /|/g;s/ * |/|/g' | paste -d'|' - clust/clusters.out.patch.5 > ${table}.db
### Parse out genus names for IDL to use later
cat ${table}.db | awk -F'|' '{print $3}' > ${fname2}
#
paste -d'|' ${table}.db clust/clusters.out.patch.10 > ${table}.db1; mv ${table}.db1 ${table}.db
#
for ((i=20;i<=100;i+=20)); do
   paste -d'|' ${table}.db clust/clusters.out.patch.$i > ${table}.db1; mv ${table}.db1 ${table}.db
done
### Parse in cluster assignments for random seed clustering trials
### for n=40 clusters
for ((i=1;i<=3;i++)); do
   paste -d'|' ${table}.db clust/clusters.out.patch.40.$i > ${table}.db1; mv ${table}.db1 ${table}.db
done
#
echo Create ${table} for IDL routines
psql sed_rock < ${table}.sql
awk -f load_patch.awk ${table}.db | psql sed_rock
echo Done
echo ""
echo Parsing queried data for IDL routines
echo Data 0-${tmax} Ma
psql sed_rock -tc "select ma_mid,latdec,londec,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth,clust_num_10,clust_num_20,clust_num_40,clust_num_60,clust_num_80,clust_num_100,clust_num_40_t1,clust_num_40_t2,clust_num_40_t3 from ${table} order by ma_mid" | awk -F'|' '{printf "%s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s\n", $1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17;}' | head -n-1 > ${fname3}
#
echo Done
echo ""
###
echo Finding assemblage diversity indexes and generating diversity maps
pushd idl/mdiv
/usr/local/itt/idl64/bin/idl << EOF
mdiv
count_map
multi_map
h_tests
exit
EOF
popd
echo Done
echo ""
echo Parsing IDL output from mdiv.pro and loading into PSQL table mdiv_meso.
paste ${fname4} ${fname5} |  awk -F' ' '{printf "%s|%s|%s|%s|%s|%s|%s|%s\n", $1,$2,$3,$4,$5,$6,$7,$8;}' > mdiv_meso.db
echo Populating table mdiv_meso
awk -f load_meso.awk mdiv_meso.db | psql sed_rock
echo Done populating table mdiv_meso
echo ""
echo Parsing IDL output from mdiv.pro and loading into PSQL table mdiv_icorr.
paste ${fname6} ${fname7} | paste - ${fname8} | paste - ${fname9} | paste - ${fname10} | paste - ${fname11} | paste - ${fname12} | awk -F' ' '{printf "%s|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f\n", $1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$20,$21,$22;}' > mdiv_icorr.db
echo Populating table mdiv_icorr
awk -f load_icorr.awk mdiv_icorr.db | psql sed_rock
echo Done populating table mdiv_icorr
echo ""
echo Parsing IDL output from mdiv.pro and loading into PSQL table mdiv_cen_corr.
paste ${fname13} ${fname14} | paste - ${fname15} | awk -F' ' '{printf "%s|%s|%s|%1.10f|%s|%s|%1.10f|%s|%s|%1.10f\n", $1,$2,$3,$4,$5,$6,$7,$8,$9,$10;}' > mdiv_cen_corr.db
awk -f load_cen_corr.awk mdiv_cen_corr.db | psql sed_rock
echo Done populating table mdiv_cen_corr
echo ""
echo All done 
echo `date`
