;
pro count_map
;
; 10 Myr increments from 0-80 Ma
age_size=[-1,29514,52260,56326,70606,79115,83211,106375,112635]
; step size
stsz=1

;Read in data
openr,1,'data/mdiv_patch.dat'
data=dblarr(17,83213)
readf,1,data
close,1

openr,1,'data/gnames.dat'
taxa=strarr(1,83213)
readf,1,taxa
close,1

; the total number of degrees
nlon=360L
nlat=180L
;print,'nlon=',nlon,' nlat=',nlat,' nlon*nlat=',nlon*nlat

set_plot,'z'
;set_plot,'ps'
loadct,27
tvlct,RedTab,GreenTab,BlueTab,/get
RedTab(1)=0
GreenTab(1)=0
BlueTab(1)=0
tvlct,RedTab,GreenTab,BlueTab
device,set_font='Helvetica',/tt_font,set_character_size=[8,12],set_graphics_function=3L,set_resolution=[400,400],z_buffering=0
;device,/encapsulated,/color,bits=24,font_size=12,/helvetica,/bold,file='images/count_map.eps'
;
num=0
for i=0,40,(stsz*10) do begin
   print,'Searching '+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'Ma'
   lat=data[ 1,age_size[num]+1:age_size[num+stsz]+1 ]
   lon=data[ 2,age_size[num]+1:age_size[num+stsz]+1 ]
   clust_num_40=data[ 10,age_size[num]+1:age_size[num+stsz]+1 ]
   names=taxa[ 0,age_size[num]+1:age_size[num+stsz]+1 ]
   ;
   ;clust_num_40=data[ 10,* ]
   ;names=taxa[ 0,* ]
   x=dindgen(nlon)-179.5
   y=dindgen(nlat)-89.5
   tc=lonarr(nlon,nlat)
   bd=lonarr(nlon,nlat)
   betai=dblarr(8)
   beta=dblarr(nlon,nlat)
   ;;; define pointer tctmp that will point to a string vector
   ;;; declare and allocate 2d array for depositional habitats
   hab40=lonarr(nlon,nlat)
   for mlon=0,nlon-1 do begin
      for mlat=0,nlat-1 do begin
;         print,x[mlon],y[mlon]
         ttmp = gstring(lon,lat,names,x[mlon],y[mlat])
         ;;;specify clust_num index with variable ctemp avoid syntax problems
         ;;;tc is the number of unique genus/class names
         idx=where(lat ge y[mlat]-0.5 and lat lt y[mlat]+0.5 and lon ge x[mlon]-0.5 and lon lt x[mlon]+0.5,cnt)
         if cnt gt 0 then begin
            tc[mlon,mlat]=n_elements(ttmp)
            ctemp_40=clust_num_40[idx]
            hab40[mlon,mlat]=n_elements(ctemp_40[uniq(ctemp_40, sort(ctemp_40))])
            ; The following routine determines the beta-diversity 
            ; (difference) between each 1-square-degree-cell and the 
            ; 8 cells surrounding it. Cells that occur on any 
            ; of the four boundaries are without a neighboring cell
            ; on either one or two sides and are therefore compared to 
            ; cells back at the beginning. For example, a cell with 
            ; longitude from -179 to -180 is compared to its counterpart 
            ; with longitude from 179 to 180. A cell at -89 to -90 
            ; latitude is compared to its counterpart at 89 to 90 
            ; latitude. Below is a grid that illustrates this method
            ; where ! is the cell of interest. Its diversity count 
            ; is compared to each of the 8 cells around it and its beta
            ; diversity is taken to be the mean of all (up to 8) of 
            ; its beta (difference) diversities provided the neighboring
            ; cell is populated by at least one occurrence.
            ;
            ; NW N NE
            ; W  !  E
            ; SW S SE
            ;
            for j=0,7 do begin
               ; Choose the lon/lat indices (iqlon,iqlat) to query for unique
               ; genus/class names
               if j lt 3 then begin
                  ; N
                  if mlat eq 0 then iqlat=nlat-1 else iqlat=mlat+1
               endif else begin
                  if j lt 5 then begin
                     iqlat = mlat
                  endif else begin
                     ; S
                     if mlat eq nlat-1 then iqlat = 0 else iqlat = mlat-1
                  endelse
               endelse
               if j eq 0 or j eq 3 or j eq 5 then begin
                  ; W
                  if mlon eq 0 then iqlon=nlon-1 else iqlon=mlon-1
               endif else begin
                  if j eq 1 or j eq 6 then begin
                     iqlon = mlon
                  endif else begin
                     ; E
                     if mlon eq nlon-1 then iqlon = 0 else iqlon = mlon+1
                  endelse
               endelse
               ; Query for unique vectors of genus/class names
               case j of
                  ; NW
                  0: ttmp_nw = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; N
                  1: ttmp_n = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; NE
                  2: ttmp_ne = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; W
                  3: ttmp_w = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; E
                  4: ttmp_e = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; SW
                  5: ttmp_sw = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; S
                  6: ttmp_s = gstring(lon,lat,names,x[iqlon],y[iqlat])
                  ; SE
                  7: ttmp_se = gstring(lon,lat,names,x[iqlon],y[iqlat])
               endcase
            endfor
            ; Compute equation for beta diversity
            for j=0,7 do begin
               case j of
                  0: ttmp_cmp = [ttmp,ttmp_nw]
                  1: ttmp_cmp = [ttmp,ttmp_n]
                  2: ttmp_cmp = [ttmp,ttmp_ne]
                  3: ttmp_cmp = [ttmp,ttmp_w]
                  4: ttmp_cmp = [ttmp,ttmp_e]
                  5: ttmp_cmp = [ttmp,ttmp_sw]
                  6: ttmp_cmp = [ttmp,ttmp_s]
                  7: ttmp_cmp = [ttmp,ttmp_se]
               endcase
               gidx = where(ttmp_cmp eq '',tcnt)
               if tcnt eq 0 then begin
                  tc_cmp = n_elements(ttmp_cmp)
                  uttmp_cmp = ttmp_cmp[uniq(ttmp_cmp, sort(ttmp_cmp))]
                  betai[j] = double(n_elements(uttmp_cmp)) / double(tc_cmp / 2.0 )
               endif else begin
                  betai[j] = 0
               endelse
            endfor
            ;beta[mlon,mlat] = max(betai)
            beta[mlon,mlat] = mean(betai,/double)
         endif else begin
            ; If no data in this gridcell, set tc and beta to zero.
            tc[mlon,mlat] = 0
            beta[mlon,mlat] = 0
         endelse
         ; Shift right/down (check to see if we're on an edge)
         ;this prints the beta diversity of each grid cell
         ;if beta[mlon,mlat] gt 0 then print,'lon=',x[mlon],' lat=',y[mlat],' alpha=',tc[mlon,mlat],' beta=',beta[mlon,mlat],' habitats=',habitats[mlon,mlat],' drange=',drange[mlon,mlat],' dmean=',dmean[mlon,mlat]
      endfor ; mlat
   endfor ; mlon
   ; declare the range of tc to pass to map_legend.
   tmin = min(tc[where(tc gt 0)])
   tmax = max(tc[where(tc gt 0)])
   ;
   tv_data=bytscl(tc,max=tmax,min=tmin,top=251)+3
   idx = where(tc eq 0,cnt)
   if cnt gt 0 then tv_data[idx] = 255
   ; Alpha Diversity Eastern Asia
   map_set,0,0,0,/miller,/isotropic,limit=[20,120,70,160],title='Alpha Diversity (East Asia)',color=1
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_continents,/grid,/coasts,color=2;,/fill
   ;map_horizon,color=1
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_adiv_eas_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
   ; Alpha Diversity Western Europe
   map_set,0,0,0,/miller,/isotropic,limit=[30,-10,60,30],title='Alpha Diversity (Western Europe)',color=1
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_continents,/grid,/coasts,color=2;,/fill
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_adiv_eur_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
   ; Alpha Diversity Southeastern US
   map_set,0,0,0,/miller,/isotropic,limit=[23,-97,45,-72],title='Alpha Diversity (Southeastern US)',color=1
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_continents,/grid,/coasts,color=2;,/fill
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_adiv_us_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
   ;
   ; declare the range of hab40 to pass to map_legend.
   hmin = min(hab40[where(hab40 gt 0)])
   hmax = max(hab40[where(hab40 gt 0)])
   ;
   tv_data=bytscl(hab40,max=hmax,min=hmin,top=251)+3
   idx = where(hab40 eq 0,cnt)
   if cnt gt 0 then tv_data[idx] = 255
   ; Depositional Habitats Western Europe
   map_set,0,0,0,/miller,/isotropic,limit=[30,-10,60,30],title='Depositional Habitats (Western Europe)',color=1
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_continents,/grid,/coasts,color=2;,/fill
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_hab40_eur_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
   ; Depositional Habitats Southeastern US
   map_set,0,0,0,/miller,/isotropic,limit=[23,-97,45,-72],title='Depositional Habitats (Southeastern US)',color=1
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_continents,/grid,/coasts,color=2;,/fill
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_hab40_us_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
;device,/encapsulated,/color,font_size=12,/helvetica,/bold,file='images/count_map_hab40_eas_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.eps'
   ; Depositional Habitats Eastern Asia
   map_set,0,0,0,/miller,/isotropic,limit=[20,120,70,160],title='Depositional Habitats (Eastern Asia)',color=1
   map_continents,/grid,/coasts,color=2,/fill
   result=map_patch(tv_data,x,y,/triangulate,max_value=254,missing=0,xstart=x0,xsize=xs,ystart=y0,ysize=ys)
   tv,result,x0,y0
   map_legend,tmin,tmax
   ;
   map=tvrd()
   write_gif,'images/count_map_hab40_eas_'+strtrim(string(i),2)+'-'+strtrim(string(i+(stsz*10)),2)+'.gif',map,RedTab,GreenTab,BlueTab
   ;
   num=num+1
   print,''
endfor
;
return
end
