pro multi_map
var_1='Alpha Diversity'
var_2='Number of Patches'
;var_3='gen-adiv'
var_3='sp-adiv'
var_4='habitats'

;Read in data
openr,1,'data/mdiv_patch.dat'
data=dblarr(14,83213)
readf,1,data
close,1

;openr,1,'data/gnames.dat'
openr,1,'data/spnames.dat'
taxa=strarr(1,83213)
readf,1,taxa
close,1

; the total number of degrees
nlon=360L
nlat=180L

set_plot,'z'
;set_plot,'ps'
loadct,27
tvlct,RedTab,GreenTab,BlueTab,/get
RedTab(1)=0
GreenTab(1)=0
BlueTab(1)=0
tvlct,RedTab,GreenTab,BlueTab
device,set_font='Helvetica Bold',/tt_font,set_character_size=[48,36],set_graphics_function=3L,set_resolution=[2000,2000],z_buffering=0
;device,/encapsulated,/color,bits=24,font_size=12,/helvetica,/bold,file='images/multi_map_'+var_3+'_'+var_4+'.eps'

;
print,'Searching entire Cenozoic'
lat=data[ 1,* ]
lon=data[ 2,* ]
clust_num_40=data[ 10,* ]
names=taxa[ 0,* ]
;;;make double precision 2d array (zc) that spans the entire globe
x=dindgen(nlon)-179.5
y=dindgen(nlat)-89.5
zc=lonarr(nlon,nlat)
tc=lonarr(nlon,nlat)
bd=lonarr(nlon,nlat)
betai=dblarr(8)
beta=dblarr(nlon,nlat)
;;; define pointer tctmp that will point to a string vector
;;; declare and allocate 2d array for depositional habitats
hab40=lonarr(nlon,nlat)
for mlon=0,nlon-1 do begin
   for mlat=0,nlat-1 do begin
;         print,x[mlon],y[mlon]
      ttmp = gstring(lon,lat,names,x[mlon],y[mlat])
      ;;;specify clust_num index with variable ctemp avoid syntax problems
      ;;;zc is the number of observations
      ;;;tc is the number of unique genus/class names
      idx=where(lat ge y[mlat]-0.5 and lat lt y[mlat]+0.5 and lon ge x[mlon]-0.5 and lon lt x[mlon]+0.5,cnt)
      if cnt gt 0 then begin
         tc[mlon,mlat]=n_elements(ttmp)
         ctemp_40=clust_num_40[idx]
         hab40[mlon,mlat]=n_elements(ctemp_40[uniq(ctemp_40, sort(ctemp_40))])
         ; The following routine determines the beta-diversity 
         ; (difference) between each 1-square-degree-cell and the 
         ; 8 cells surrounding it. Cells that occur on any 
         ; of the four boundaries are without a neighboring cell
         ; on either one or two sides and are therefore compared to 
         ; cells back at the beginning. For example, a cell with 
         ; longitude from -179 to -180 is compared to its counterpart 
         ; with longitude from 179 to 180. A cell at -89 to -90 
         ; latitude is compared to its counterpart at 89 to 90 
         ; latitude. Below is a grid that illustrates this method
         ; where ! is the cell of interest. Its diversity count 
         ; is compared to each of the 8 cells around it and its beta
         ; diversity is taken to be the mean of all (up to 8) of 
         ; its beta (difference) diversities provided the neighboring
         ; cell is populated by at least one occurrence.
         ;
         ; NW N NE
         ; W  !  E
         ; SW S SE
         ;
         for j=0,7 do begin
            ; Choose the lon/lat indices (iqlon,iqlat) to query for unique
            ; genus/class names
            if j lt 3 then begin
               ; N
               if mlat eq 0 then iqlat=nlat-1 else iqlat=mlat+1
            endif else begin
               if j lt 5 then begin
                  iqlat = mlat
               endif else begin
                  ; S
                  if mlat eq nlat-1 then iqlat = 0 else iqlat = mlat-1
               endelse
            endelse
            if j eq 0 or j eq 3 or j eq 5 then begin
               ; W
               if mlon eq 0 then iqlon=nlon-1 else iqlon=mlon-1
            endif else begin
               if j eq 1 or j eq 6 then begin
                  iqlon = mlon
               endif else begin
                  ; E
                  if mlon eq nlon-1 then iqlon = 0 else iqlon = mlon+1
               endelse
            endelse
            ; Query for unique vectors of genus/class names
            case j of
               ; NW
               0: ttmp_nw = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; N
               1: ttmp_n = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; NE
               2: ttmp_ne = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; W
               3: ttmp_w = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; E
               4: ttmp_e = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; SW
               5: ttmp_sw = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; S
               6: ttmp_s = gstring(lon,lat,names,x[iqlon],y[iqlat])
               ; SE
               7: ttmp_se = gstring(lon,lat,names,x[iqlon],y[iqlat])
            endcase
         endfor
         ; Compute equation for beta diversity
         for j=0,7 do begin
            case j of
               0: ttmp_cmp = [ttmp,ttmp_nw]
               1: ttmp_cmp = [ttmp,ttmp_n]
               2: ttmp_cmp = [ttmp,ttmp_ne]
               3: ttmp_cmp = [ttmp,ttmp_w]
               4: ttmp_cmp = [ttmp,ttmp_e]
               5: ttmp_cmp = [ttmp,ttmp_sw]
               6: ttmp_cmp = [ttmp,ttmp_s]
               7: ttmp_cmp = [ttmp,ttmp_se]
            endcase
            gidx = where(ttmp_cmp eq '',tcnt)
            if tcnt eq 0 then begin
               tc_cmp = n_elements(ttmp_cmp)
               uttmp_cmp = ttmp_cmp[uniq(ttmp_cmp, sort(ttmp_cmp))]
               betai[j] = double(n_elements(uttmp_cmp)) / double(tc_cmp / 2.0 )
            endif else begin
               betai[j] = 0
            endelse
         endfor
         ;beta[mlon,mlat] = max(betai)
         beta[mlon,mlat] = mean(betai,/double)
      endif else begin
         ; If no data in this gridcell, set tc and beta to zero.
         tc[mlon,mlat] = 0
         beta[mlon,mlat] = 0
      endelse
      ; Shift right/down (check to see if we're on an edge)
      ;if cnt gt 0 then begin
      ;   print,'tc',tc[mlon,mlat],' drange',drange[mlon,mlat]
      ;endif
      ;this prints the beta diversity of each grid cell
      ;if beta[mlon,mlat] gt 0 then print,'lon=',x[mlon],' lat=',y[mlat],' alpha=',tc[mlon,mlat],' beta=',beta[mlon,mlat],' habitats=',habitats[mlon,mlat],' drange=',drange[mlon,mlat],' dmean=',dmean[mlon,mlat]
   endfor ; mlat
endfor ; mlon
;write data
;printf,1,foo_alpha,foo_beta,foo_habitats,format='(f,3x,f,3x,f,3x)'
;use nx & ny to set the dimensions of GRIDDATA
nx=n_elements(x)
ny=n_elements(y)
;;;print,'X is',x,'Y is',y
if n_elements(x) eq n_elements(tc) then x = x $
else if n_elements(x) eq nx then x = x # replicate(1., ny) $
else message, 'Dimension of LON incompatible'
if n_elements(y) eq n_elements(tc) then y = y $
else if n_elements(y) eq ny then y = replicate(1.,nx) # y $
else message, 'Dimension of LAT incompatible'
qhull,x,y,tris,sphere=s
s=0
surfdata=griddata(x,y,tc,/degrees,/sphere,/nearest_neighbor,triangles=tris,dimension=[nx,ny]);,empty_sectors=2)
colordata=griddata(x,y,hab40,/degrees,/sphere,/nearest_neighbor,triangles=tris,dimension=[nx,ny]);,empty_sectors=2)
;
; draw map for globe
shade_surf,surfdata,shades=bytscl(colordata),pixels=600,position=[0.3, 0.25, 0.95, 1.0],xtitle='Longitude',ytitle='Latitude',ztitle=var_1,xrange=[0,360],yrange=[0,180],zrange=[1,2600],xtickinterval=90,ytickinterval=45,ztickinterval=400,xminor=-1,yminor=-1,zminor=-1,xthick=6,ythick=6,zthick=6,xstyle=1,ystyle=1,zstyle=1,color=1,charthick=6,charsize=2.0,ymargin=[4,3];ax=50,
;shade_surf,surfdata,shades=bytscl(colordata),pixels=600,position=[0.15, 0.15, 0.95, 0.8],xtitle='Longitude',ytitle='Latitude',ztitle=var_1,xrange=[0,360],yrange=[0,180],zrange=[1,1000],xtickinterval=90,ytickinterval=45,xthick=4,ythick=4,zthick=4,xstyle=1,ystyle=1,zstyle=1,color=1,charthick=4,charsize=2;ax=50,
;
; declare the range of hab40 to pass to map_legend.
hab40min = min(hab40[where(hab40 gt 0)])
hab40max = max(hab40[where(hab40 gt 0)])
;
map_legend,hab40min,hab40max
map=tvrd()
write_gif,'images/multi_map_global_'+var_3+'_'+var_4+'.gif',map,RedTab,GreenTab,BlueTab
;
;
return
end
